@@ -82,7 +82,7 @@ interaction_plot <- function(means,
         ifelse(is.list(this), 1L + 
                  as.numeric(all(as.logical(sapply(this, depth)))), 0L)
       }
-      if (depth(data_arg) == 1) {
+      if ((length(data_arg) == 0) | (depth(data_arg) == 1)) {
         if (!is.null(data_alpha)) data_arg$alpha <- data_alpha
         if (!is.null(data_color) & !any(c("color", "colour") %in% mapping)) {
           data_arg$color <- data_color
@@ -203,7 +203,8 @@ interaction_plot <- function(means,
     plot_out <- plot_out + 
       ggplot2::xlab(attr(means, "x"))
   }
-  if (!missing(legend_title)) {
+  mapping <- mapping[nzchar(mapping)]
+  if (!missing(legend_title) && length(mapping) > 0) {
     legend_title <- paste(legend_title, collapse = "\n")
     tmp_list <- rep(list(ggplot2::guide_legend(title = legend_title)), 
                     length(mapping))
@@ -362,7 +363,8 @@ oneway_plot <- function(means,
     plot_out <- plot_out + 
       ggplot2::xlab(attr(means, "x"))
   }
-  if (!missing(legend_title)) {
+  mapping <- mapping[nzchar(mapping)]
+  if (!missing(legend_title) && length(mapping) > 0) {
     legend_title <- paste(legend_title, collapse = "\n")
     tmp_list <- rep(list(ggplot2::guide_legend(title = legend_title)), 
                     length(mapping))
